@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "03/05/2018"
+__date__ = "12/03/2019"
 
 
 import time
@@ -43,10 +43,7 @@ from silx.gui.utils.testutils import SignalListener
 from silx.io import commonh5
 import weakref
 
-try:
-    import h5py
-except ImportError:
-    h5py = None
+import h5py
 
 
 _tmpDirectory = None
@@ -56,14 +53,13 @@ def setUpModule():
     global _tmpDirectory
     _tmpDirectory = tempfile.mkdtemp(prefix=__name__)
 
-    if h5py is not None:
-        filename = _tmpDirectory + "/data.h5"
+    filename = _tmpDirectory + "/data.h5"
 
-        # create h5 data
-        f = h5py.File(filename, "w")
-        g = f.create_group("arrays")
-        g.create_dataset("scalar", data=10)
-        f.close()
+    # create h5 data
+    f = h5py.File(filename, "w")
+    g = f.create_group("arrays")
+    g.create_dataset("scalar", data=10)
+    f.close()
 
 
 def tearDownModule():
@@ -91,8 +87,6 @@ class TestHdf5TreeModel(TestCaseQt):
 
     def setUp(self):
         super(TestHdf5TreeModel, self).setUp()
-        if h5py is None:
-            self.skipTest("h5py is not available")
 
     def waitForPendingOperations(self, model):
         for _ in range(10):
@@ -127,8 +121,6 @@ class TestHdf5TreeModel(TestCaseQt):
         model.appendFile(filename)
         self.assertEqual(model.rowCount(qt.QModelIndex()), 1)
         # clean up
-        index = model.index(0, 0, qt.QModelIndex())
-        h5File = model.data(index, hdf5.Hdf5TreeModel.H5PY_OBJECT_ROLE)
         ref = weakref.ref(model)
         model = None
         self.qWaitForDestroy(ref)
@@ -232,7 +224,7 @@ class TestHdf5TreeModel(TestCaseQt):
 
     def testSupportedDrop(self):
         model = hdf5.Hdf5TreeModel()
-        self.assertNotEquals(model.supportedDropActions(), 0)
+        self.assertNotEqual(model.supportedDropActions(), 0)
 
         model.setFileMoveEnabled(False)
         model.setFileDropEnabled(False)
@@ -240,11 +232,42 @@ class TestHdf5TreeModel(TestCaseQt):
 
         model.setFileMoveEnabled(False)
         model.setFileDropEnabled(True)
-        self.assertNotEquals(model.supportedDropActions(), 0)
+        self.assertNotEqual(model.supportedDropActions(), 0)
 
         model.setFileMoveEnabled(True)
         model.setFileDropEnabled(False)
-        self.assertNotEquals(model.supportedDropActions(), 0)
+        self.assertNotEqual(model.supportedDropActions(), 0)
+
+    def testCloseFile(self):
+        """A file inserted as a filename is open and closed internally."""
+        filename = _tmpDirectory + "/data.h5"
+        model = hdf5.Hdf5TreeModel()
+        self.assertEqual(model.rowCount(qt.QModelIndex()), 0)
+        model.insertFile(filename)
+        self.assertEqual(model.rowCount(qt.QModelIndex()), 1)
+        index = model.index(0, 0)
+        h5File = model.data(index, role=hdf5.Hdf5TreeModel.H5PY_OBJECT_ROLE)
+        model.removeIndex(index)
+        self.assertEqual(model.rowCount(qt.QModelIndex()), 0)
+        self.assertFalse(bool(h5File.id.valid), "The HDF5 file was not closed")
+
+    def testNotCloseFile(self):
+        """A file inserted as an h5py object is not open (then not closed)
+        internally."""
+        filename = _tmpDirectory + "/data.h5"
+        try:
+            h5File = h5py.File(filename)
+            model = hdf5.Hdf5TreeModel()
+            self.assertEqual(model.rowCount(qt.QModelIndex()), 0)
+            model.insertH5pyObject(h5File)
+            self.assertEqual(model.rowCount(qt.QModelIndex()), 1)
+            index = model.index(0, 0)
+            h5File = model.data(index, role=hdf5.Hdf5TreeModel.H5PY_OBJECT_ROLE)
+            model.removeIndex(index)
+            self.assertEqual(model.rowCount(qt.QModelIndex()), 0)
+            self.assertTrue(bool(h5File.id.valid), "The HDF5 file was unexpetedly closed")
+        finally:
+            h5File.close()
 
     def testDropExternalFile(self):
         filename = _tmpDirectory + "/data.h5"
@@ -571,8 +594,6 @@ class TestH5Node(TestCaseQt):
     @classmethod
     def setUpClass(cls):
         super(TestH5Node, cls).setUpClass()
-        if h5py is None:
-            raise unittest.SkipTest("h5py is not available")
 
         cls.tmpDirectory = tempfile.mkdtemp()
         cls.h5Filename = cls.createResource(cls.tmpDirectory)
@@ -809,8 +830,6 @@ class TestHdf5TreeView(TestCaseQt):
 
     def setUp(self):
         super(TestHdf5TreeView, self).setUp()
-        if h5py is None:
-            self.skipTest("h5py is not available")
 
     def testCreate(self):
         view = hdf5.Hdf5TreeView()
