@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2014-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,11 +28,12 @@ from __future__ import division
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "01/08/2018"
+__date__ = "21/12/2018"
 
 from collections import OrderedDict, namedtuple
-from ctypes import c_void_p
 import logging
+import warnings
+import weakref
 
 import numpy
 
@@ -44,10 +45,11 @@ from ... import qt
 from ..._glutils import gl
 from ... import _glutils as glu
 from .glutils import (
+    GLLines2D, GLPlotTriangles,
     GLPlotCurve2D, GLPlotColormap, GLPlotRGBAImage, GLPlotFrame2D,
     mat4Ortho, mat4Identity,
     LEFT, RIGHT, BOTTOM, TOP,
-    Text2D, Shape2D)
+    Text2D, FilledShape2D)
 from .glutils.PlotImageFile import saveImageToFile
 
 _logger = logging.getLogger(__name__)
@@ -105,7 +107,7 @@ class PlotDataContent(object):
     This class is only meant to work with _OpenGLPlotCanvas.
     """
 
-    _PRIMITIVE_TYPES = 'curve', 'image'
+    _PRIMITIVE_TYPES = 'curve', 'image', 'triangles'
 
     def __init__(self):
         self._primitives = OrderedDict()  # For images and curves
@@ -123,6 +125,8 @@ class PlotDataContent(object):
             primitiveType = 'curve'
         elif isinstance(primitive, (GLPlotColormap, GLPlotRGBAImage)):
             primitiveType = 'image'
+        elif isinstance(primitive, GLPlotTriangles):
+            primitiveType = 'triangles'
         else:
             raise RuntimeError('Unsupported object type: %s', primitive)
 
@@ -303,16 +307,8 @@ _texFragShd = """
     }
     """
 
-
 # BackendOpenGL ###############################################################
 
-_current_context = None
-
-
-def _getContext():
-    assert _current_context is not None
-    return _current_context
-
 
 class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
     """OpenGL-based Plot backend.
@@ -338,13 +334,16 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                                   f=f)
         BackendBase.BackendBase.__init__(self, plot, parent)
 
+        self._backgroundColor = 1., 1., 1., 1.
+        self._dataBackgroundColor = 1., 1., 1., 1.
+
         self.matScreenProj = mat4Identity()
 
         self._progBase = glu.Program(
             _baseVertShd, _baseFragShd, attrib0='position')
         self._progTex = glu.Program(
             _texVertShd, _texFragShd, attrib0='position')
-        self._plotFBOs = {}
+        self._plotFBOs = weakref.WeakKeyDictionary()
 
         self._keepDataAspectRatio = False
 
@@ -357,6 +356,8 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         self._glGarbageCollector = []
 
         self._plotFrame = GLPlotFrame2D(
+            foregroundColor=(0., 0., 0., 1.),
+            gridColor=(.7, .7, .7, 1.),
             margins={'left': 100, 'right': 50, 'top': 50, 'bottom': 50})
 
         # Make postRedisplay asynchronous using Qt signal
@@ -380,6 +381,8 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         return qt.QSize(8 * 80, 6 * 80)  # Mimic MatplotlibBackend
 
     def mousePressEvent(self, event):
+        if event.button() not in self._MOUSE_BTNS:
+            return super(BackendOpenGL, self).mousePressEvent(event)
         xPixel = event.x() * self.getDevicePixelRatio()
         yPixel = event.y() * self.getDevicePixelRatio()
         btn = self._MOUSE_BTNS[event.button()]
@@ -405,6 +408,8 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         event.accept()
 
     def mouseReleaseEvent(self, event):
+        if event.button() not in self._MOUSE_BTNS:
+            return super(BackendOpenGL, self).mouseReleaseEvent(event)
         xPixel = event.x() * self.getDevicePixelRatio()
         yPixel = event.y() * self.getDevicePixelRatio()
 
@@ -432,7 +437,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
     def initializeGL(self):
         gl.testGL()
 
-        gl.glClearColor(1., 1., 1., 1.)
         gl.glClearStencil(0)
 
         gl.glEnable(gl.GL_BLEND)
@@ -457,15 +461,17 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         self._renderOverlayGL()
 
     def _paintFBOGL(self):
-        context = glu.getGLContext()
+        context = glu.Context.getCurrent()
         plotFBOTex = self._plotFBOs.get(context)
         if (self._plot._getDirtyPlot() or self._plotFrame.isDirty or
                 plotFBOTex is None):
-            self._plotVertices = numpy.array(((-1., -1., 0., 0.),
-                                             (1., -1., 1., 0.),
-                                             (-1., 1., 0., 1.),
-                                             (1., 1., 1., 1.)),
-                                             dtype=numpy.float32)
+            self._plotVertices = (
+                # Vertex coordinates
+                numpy.array(((-1., -1.), (1., -1.), (-1., 1.), (1., 1.)),
+                             dtype=numpy.float32),
+                 # Texture coordinates
+                 numpy.array(((0., 0.), (1., 0.), (0., 1.), (1., 1.)),
+                             dtype=numpy.float32))
             if plotFBOTex is None or \
                plotFBOTex.shape[1] != self._plotFrame.size[0] or \
                plotFBOTex.shape[0] != self._plotFrame.size[1]:
@@ -482,6 +488,7 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                 self._plotFBOs[context] = plotFBOTex
 
             with plotFBOTex:
+                gl.glClearColor(*self._backgroundColor)
                 gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_STENCIL_BUFFER_BIT)
                 self._renderPlotAreaGL()
                 self._plotFrame.render()
@@ -496,146 +503,45 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         gl.glUniformMatrix4fv(self._progTex.uniforms['matrix'], 1, gl.GL_TRUE,
                               mat4Identity().astype(numpy.float32))
 
-        stride = self._plotVertices.shape[-1] * self._plotVertices.itemsize
         gl.glEnableVertexAttribArray(self._progTex.attributes['position'])
         gl.glVertexAttribPointer(self._progTex.attributes['position'],
                                  2,
                                  gl.GL_FLOAT,
                                  gl.GL_FALSE,
-                                 stride, self._plotVertices)
+                                 0,
+                                 self._plotVertices[0])
 
-        texCoordsPtr = c_void_p(self._plotVertices.ctypes.data +
-                                2 * self._plotVertices.itemsize)  # Better way?
         gl.glEnableVertexAttribArray(self._progTex.attributes['texCoords'])
         gl.glVertexAttribPointer(self._progTex.attributes['texCoords'],
                                  2,
                                  gl.GL_FLOAT,
                                  gl.GL_FALSE,
-                                 stride, texCoordsPtr)
+                                 0,
+                                 self._plotVertices[1])
 
         with plotFBOTex.texture:
-            gl.glDrawArrays(gl.GL_TRIANGLE_STRIP, 0, len(self._plotVertices))
+            gl.glDrawArrays(gl.GL_TRIANGLE_STRIP, 0, len(self._plotVertices[0]))
 
         self._renderMarkersGL()
         self._renderOverlayGL()
 
     def paintGL(self):
-        global _current_context
-        _current_context = self.context()
+        with glu.Context.current(self.context()):
+            # Release OpenGL resources
+            for item in self._glGarbageCollector:
+                item.discard()
+            self._glGarbageCollector = []
 
-        glu.setGLContextGetter(_getContext)
+            gl.glClearColor(*self._backgroundColor)
+            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_STENCIL_BUFFER_BIT)
 
-        # Release OpenGL resources
-        for item in self._glGarbageCollector:
-            item.discard()
-        self._glGarbageCollector = []
+            # Check if window is large enough
+            plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]
+            if plotWidth <= 2 or plotHeight <= 2:
+                return
 
-        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_STENCIL_BUFFER_BIT)
-
-        # Check if window is large enough
-        plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]
-        if plotWidth <= 2 or plotHeight <= 2:
-            return
-
-        # self._paintDirectGL()
-        self._paintFBOGL()
-
-        glu.setGLContextGetter()
-        _current_context = None
-
-    def _nonOrthoAxesLineMarkerPrimitives(self, marker, pixelOffset):
-        """Generates the vertices and label for a line marker.
-
-        :param dict marker: Description of a line marker
-        :param int pixelOffset: Offset of text from borders in pixels
-        :return: Line vertices and Text label or None
-        :rtype: 2-tuple (2x2 numpy.array of float, Text2D)
-        """
-        label, vertices = None, None
-
-        xCoord, yCoord = marker['x'], marker['y']
-        assert xCoord is None or yCoord is None  # Specific to line markers
-
-        # Get plot corners in data coords
-        plotLeft, plotTop, plotWidth, plotHeight = self.getPlotBoundsInPixels()
-
-        corners = [(plotLeft, plotTop),
-                   (plotLeft, plotTop + plotHeight),
-                   (plotLeft + plotWidth, plotTop + plotHeight),
-                   (plotLeft + plotWidth, plotTop)]
-        corners = numpy.array([self.pixelToData(x, y, axis='left', check=False)
-                               for (x, y) in corners])
-
-        borders = {
-            'right': (corners[3], corners[2]),
-            'top': (corners[0], corners[3]),
-            'bottom': (corners[2], corners[1]),
-            'left': (corners[1], corners[0])
-        }
-
-        textLayouts = {  # align, valign, offsets
-            'right': (RIGHT, BOTTOM, (-1., -1.)),
-            'top': (LEFT, TOP, (1., 1.)),
-            'bottom': (LEFT, BOTTOM, (1., -1.)),
-            'left': (LEFT, BOTTOM, (1., -1.))
-        }
-
-        if xCoord is None:  # Horizontal line in data space
-            if marker['text'] is not None:
-                # Find intersection of hline with borders in data
-                # Order is important as it stops at first intersection
-                for border_name in ('right', 'top', 'bottom', 'left'):
-                    (x0, y0), (x1, y1) = borders[border_name]
-
-                    if min(y0, y1) <= yCoord < max(y0, y1):
-                        xIntersect = (yCoord - y0) * (x1 - x0) / (y1 - y0) + x0
-
-                        # Add text label
-                        pixelPos = self.dataToPixel(
-                            xIntersect, yCoord, axis='left', check=False)
-
-                        align, valign, offsets = textLayouts[border_name]
-
-                        x = pixelPos[0] + offsets[0] * pixelOffset
-                        y = pixelPos[1] + offsets[1] * pixelOffset
-                        label = Text2D(marker['text'], x, y,
-                                       color=marker['color'],
-                                       bgColor=(1., 1., 1., 0.5),
-                                       align=align, valign=valign)
-                        break  # Stop at first intersection
-
-            xMin, xMax = corners[:, 0].min(), corners[:, 0].max()
-            vertices = numpy.array(
-                ((xMin, yCoord), (xMax, yCoord)), dtype=numpy.float32)
-
-        else:  # yCoord is None: vertical line in data space
-            if marker['text'] is not None:
-                # Find intersection of hline with borders in data
-                # Order is important as it stops at first intersection
-                for border_name in ('top', 'bottom', 'right', 'left'):
-                    (x0, y0), (x1, y1) = borders[border_name]
-                    if min(x0, x1) <= xCoord < max(x0, x1):
-                        yIntersect = (xCoord - x0) * (y1 - y0) / (x1 - x0) + y0
-
-                        # Add text label
-                        pixelPos = self.dataToPixel(
-                            xCoord, yIntersect, axis='left', check=False)
-
-                        align, valign, offsets = textLayouts[border_name]
-
-                        x = pixelPos[0] + offsets[0] * pixelOffset
-                        y = pixelPos[1] + offsets[1] * pixelOffset
-                        label = Text2D(marker['text'], x, y,
-                                       color=marker['color'],
-                                       bgColor=(1., 1., 1., 0.5),
-                                       align=align, valign=valign)
-                        break  # Stop at first intersection
-
-            yMin, yMax = corners[:, 1].min(), corners[:, 1].max()
-            vertices = numpy.array(
-                ((xCoord, yMin), (xCoord, yMax)), dtype=numpy.float32)
-
-        return vertices, label
+            # self._paintDirectGL()
+            self._paintFBOGL()
 
     def _renderMarkersGL(self):
         if len(self._markers) == 0:
@@ -651,16 +557,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
 
         gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])
 
-        # Prepare vertical and horizontal markers rendering
-        self._progBase.use()
-        gl.glUniformMatrix4fv(
-            self._progBase.uniforms['matrix'], 1, gl.GL_TRUE,
-            self.matScreenProj.astype(numpy.float32))
-        gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
-        gl.glUniform1i(self._progBase.uniforms['hatchStep'], 0)
-        gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)
-        posAttrib = self._progBase.attributes['position']
-
         labels = []
         pixelOffset = 3
 
@@ -677,59 +573,43 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                 continue
 
             if xCoord is None or yCoord is None:
-                if not self.isDefaultBaseVectors():  # Non-orthogonal axes
-                    vertices, label = self._nonOrthoAxesLineMarkerPrimitives(
-                        marker, pixelOffset)
-                    if label is not None:
+                pixelPos = self.dataToPixel(
+                    xCoord, yCoord, axis='left', check=False)
+
+                if xCoord is None:  # Horizontal line in data space
+                    if marker['text'] is not None:
+                        x = self._plotFrame.size[0] - \
+                            self._plotFrame.margins.right - pixelOffset
+                        y = pixelPos[1] - pixelOffset
+                        label = Text2D(marker['text'], x, y,
+                                       color=marker['color'],
+                                       bgColor=(1., 1., 1., 0.5),
+                                       align=RIGHT, valign=BOTTOM)
                         labels.append(label)
 
-                else:  # Orthogonal axes
-                    pixelPos = self.dataToPixel(
-                        xCoord, yCoord, axis='left', check=False)
+                    width = self._plotFrame.size[0]
+                    lines = GLLines2D((0, width), (pixelPos[1], pixelPos[1]),
+                                      style=marker['linestyle'],
+                                      color=marker['color'],
+                                      width=marker['linewidth'])
+                    lines.render(self.matScreenProj)
 
-                    if xCoord is None:  # Horizontal line in data space
-                        if marker['text'] is not None:
-                            x = self._plotFrame.size[0] - \
-                                self._plotFrame.margins.right - pixelOffset
-                            y = pixelPos[1] - pixelOffset
-                            label = Text2D(marker['text'], x, y,
-                                           color=marker['color'],
-                                           bgColor=(1., 1., 1., 0.5),
-                                           align=RIGHT, valign=BOTTOM)
-                            labels.append(label)
+                else:  # yCoord is None: vertical line in data space
+                    if marker['text'] is not None:
+                        x = pixelPos[0] + pixelOffset
+                        y = self._plotFrame.margins.top + pixelOffset
+                        label = Text2D(marker['text'], x, y,
+                                       color=marker['color'],
+                                       bgColor=(1., 1., 1., 0.5),
+                                       align=LEFT, valign=TOP)
+                        labels.append(label)
 
-                        width = self._plotFrame.size[0]
-                        vertices = numpy.array(((0, pixelPos[1]),
-                                                (width, pixelPos[1])),
-                                               dtype=numpy.float32)
-
-                    else:  # yCoord is None: vertical line in data space
-                        if marker['text'] is not None:
-                            x = pixelPos[0] + pixelOffset
-                            y = self._plotFrame.margins.top + pixelOffset
-                            label = Text2D(marker['text'], x, y,
-                                           color=marker['color'],
-                                           bgColor=(1., 1., 1., 0.5),
-                                           align=LEFT, valign=TOP)
-                            labels.append(label)
-
-                        height = self._plotFrame.size[1]
-                        vertices = numpy.array(((pixelPos[0], 0),
-                                                (pixelPos[0], height)),
-                                               dtype=numpy.float32)
-
-                self._progBase.use()
-                gl.glUniform4f(self._progBase.uniforms['color'],
-                               *marker['color'])
-
-                gl.glEnableVertexAttribArray(posAttrib)
-                gl.glVertexAttribPointer(posAttrib,
-                                         2,
-                                         gl.GL_FLOAT,
-                                         gl.GL_FALSE,
-                                         0, vertices)
-                gl.glLineWidth(1)
-                gl.glDrawArrays(gl.GL_LINES, 0, len(vertices))
+                    height = self._plotFrame.size[1]
+                    lines = GLLines2D((pixelPos[0], pixelPos[0]), (0, height),
+                                      style=marker['linestyle'],
+                                      color=marker['color'],
+                                      width=marker['linewidth'])
+                    lines.render(self.matScreenProj)
 
             else:
                 pixelPos = self.dataToPixel(
@@ -820,13 +700,17 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
     def _renderPlotAreaGL(self):
         plotWidth, plotHeight = self.getPlotBoundsInPixels()[2:]
 
-        self._plotFrame.renderGrid()
-
         gl.glScissor(self._plotFrame.margins.left,
                      self._plotFrame.margins.bottom,
                      plotWidth, plotHeight)
         gl.glEnable(gl.GL_SCISSOR_TEST)
 
+        if self._dataBackgroundColor != self._backgroundColor:
+            gl.glClearColor(*self._dataBackgroundColor)
+            gl.glClear(gl.GL_COLOR_BUFFER_BIT)
+
+        self._plotFrame.renderGrid()
+
         # Matrix
         trBounds = self._plotFrame.transformedDataRanges
         if trBounds.x[0] == trBounds.x[1] or \
@@ -853,32 +737,61 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         # Render Items
         gl.glViewport(0, 0, self._plotFrame.size[0], self._plotFrame.size[1])
 
-        self._progBase.use()
-        gl.glUniformMatrix4fv(self._progBase.uniforms['matrix'], 1, gl.GL_TRUE,
-                              self.matScreenProj.astype(numpy.float32))
-        gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
-        gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)
-
         for item in self._items.values():
             if ((isXLog and numpy.min(item['x']) < FLOAT32_MINPOS) or
                     (isYLog and numpy.min(item['y']) < FLOAT32_MINPOS)):
                 # Ignore items <= 0. on log axes
                 continue
 
-            closed = item['shape'] != 'polylines'
-            points = [self.dataToPixel(x, y, axis='left', check=False)
-                      for (x, y) in zip(item['x'], item['y'])]
-            shape2D = Shape2D(points,
-                              fill=item['fill'],
-                              fillColor=item['color'],
-                              stroke=True,
-                              strokeColor=item['color'],
-                              strokeClosed=closed)
+            if item['shape'] == 'hline':
+                width = self._plotFrame.size[0]
+                _, yPixel = self.dataToPixel(
+                    None, item['y'], axis='left', check=False)
+                points = numpy.array(((0., yPixel), (width, yPixel)),
+                                     dtype=numpy.float32)
 
-            posAttrib = self._progBase.attributes['position']
-            colorUnif = self._progBase.uniforms['color']
-            hatchStepUnif = self._progBase.uniforms['hatchStep']
-            shape2D.render(posAttrib, colorUnif, hatchStepUnif)
+            elif item['shape'] == 'vline':
+                xPixel, _ = self.dataToPixel(
+                    item['x'], None, axis='left', check=False)
+                height = self._plotFrame.size[1]
+                points = numpy.array(((xPixel, 0), (xPixel, height)),
+                                     dtype=numpy.float32)
+
+            else:
+                points = numpy.array([
+                    self.dataToPixel(x, y, axis='left', check=False)
+                    for (x, y) in zip(item['x'], item['y'])])
+
+            # Draw the fill
+            if (item['fill'] is not None and
+                    item['shape'] not in ('hline', 'vline')):
+                self._progBase.use()
+                gl.glUniformMatrix4fv(
+                    self._progBase.uniforms['matrix'], 1, gl.GL_TRUE,
+                    self.matScreenProj.astype(numpy.float32))
+                gl.glUniform2i(self._progBase.uniforms['isLog'], False, False)
+                gl.glUniform1f(self._progBase.uniforms['tickLen'], 0.)
+
+                shape2D = FilledShape2D(
+                    points, style=item['fill'], color=item['color'])
+                shape2D.render(
+                    posAttrib=self._progBase.attributes['position'],
+                    colorUnif=self._progBase.uniforms['color'],
+                    hatchStepUnif=self._progBase.uniforms['hatchStep'])
+
+            # Draw the stroke
+            if item['linestyle'] not in ('', ' ', None):
+                if item['shape'] != 'polylines':
+                    # close the polyline
+                    points = numpy.append(points,
+                                          numpy.atleast_2d(points[0]), axis=0)
+
+                lines = GLLines2D(points[:, 0], points[:, 1],
+                                  style=item['linestyle'],
+                                  color=item['color'],
+                                  dash2ndColor=item['linebgcolor'],
+                                  width=item['linewidth'])
+                lines.render(self.matScreenProj)
 
         gl.glDisable(gl.GL_SCISSOR_TEST)
 
@@ -972,7 +885,10 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                     xErrorMinus, xErrorPlus = xerror[0], xerror[1]
                 else:
                     xErrorMinus, xErrorPlus = xerror, xerror
-                xErrorMinus = logX - numpy.log10(x - xErrorMinus)
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', category=RuntimeWarning)
+                    # Ignore divide by zero, invalid value encountered in log10
+                    xErrorMinus = logX - numpy.log10(x - xErrorMinus)
                 xErrorPlus = numpy.log10(x + xErrorPlus) - logX
                 xerror = numpy.array((xErrorMinus, xErrorPlus),
                                      dtype=numpy.float32)
@@ -992,7 +908,10 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                     yErrorMinus, yErrorPlus = yerror[0], yerror[1]
                 else:
                     yErrorMinus, yErrorPlus = yerror, yerror
-                yErrorMinus = logY - numpy.log10(y - yErrorMinus)
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', category=RuntimeWarning)
+                    # Ignore divide by zero, invalid value encountered in log10
+                    yErrorMinus = logY - numpy.log10(y - yErrorMinus)
                 yErrorPlus = numpy.log10(y + yErrorPlus) - logY
                 yerror = numpy.array((yErrorMinus, yErrorPlus),
                                      dtype=numpy.float32)
@@ -1123,7 +1042,27 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
 
         return legend, 'image'
 
-    def addItem(self, x, y, legend, shape, color, fill, overlay, z):
+    def addTriangles(self, x, y, triangles, legend,
+                     color, z, selectable, alpha):
+
+        # Handle axes log scale: convert data
+        if self._plotFrame.xAxis.isLog:
+            x = numpy.log10(x)
+        if self._plotFrame.yAxis.isLog:
+            y = numpy.log10(y)
+
+        triangles = GLPlotTriangles(x, y, color, triangles, alpha)
+        triangles.info = {
+            'legend': legend,
+            'zOrder': z,
+            'behaviors': set(['selectable']) if selectable else set(),
+        }
+        self._plotContent.add(triangles)
+
+        return legend, 'triangles'
+
+    def addItem(self, x, y, legend, shape, color, fill, overlay, z,
+                linestyle, linewidth, linebgcolor):
         # TODO handle overlay
         if shape not in ('polygon', 'rectangle', 'line',
                          'vline', 'hline', 'polylines'):
@@ -1154,7 +1093,10 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
             'color': colors.rgba(color),
             'fill': 'hatch' if fill else None,
             'x': x,
-            'y': y
+            'y': y,
+            'linestyle': linestyle,
+            'linewidth': linewidth,
+            'linebgcolor': linebgcolor,
         }
 
         return legend, 'item'
@@ -1166,10 +1108,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         if symbol is None:
             symbol = '+'
 
-        if linestyle != '-' or linewidth != 1:
-            _logger.warning(
-                'OpenGL backend does not support marker line style and width.')
-
         behaviors = set()
         if selectable:
             behaviors.add('selectable')
@@ -1191,6 +1129,8 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
             'behaviors': behaviors,
             'constraint': constraint if isConstraint else None,
             'symbol': symbol,
+            'linestyle': linestyle,
+            'linewidth': linewidth,
         }
 
         return legend, 'marker'
@@ -1210,10 +1150,10 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
 
                 self._glGarbageCollector.append(curve)
 
-        elif kind == 'image':
-            image = self._plotContent.pop('image', legend)
-            if image is not None:
-                self._glGarbageCollector.append(image)
+        elif kind in ('image', 'triangles'):
+            item = self._plotContent.pop(kind, legend)
+            if item is not None:
+                self._glGarbageCollector.append(item)
 
         elif kind == 'marker':
             self._markers.pop(legend, False)
@@ -1266,6 +1206,60 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
             self._plotFrame.size[1] - self._plotFrame.margins.bottom - 1)
         return xPlot, yPlot
 
+    def __pickCurves(self, item, x, y):
+        """Perform picking on a curve item.
+
+        :param GLPlotCurve2D item:
+        :param float x: X position of the mouse in widget coordinates
+        :param float y: Y position of the mouse in widget coordinates
+        :return: List of indices of picked points
+        :rtype: List[int]
+        """
+        offset = self._PICK_OFFSET
+        if item.marker is not None:
+            offset = max(item.markerSize / 2., offset)
+        if item.lineStyle is not None:
+            offset = max(item.lineWidth / 2., offset)
+
+        yAxis = item.info['yAxis']
+
+        inAreaPos = self._mouseInPlotArea(x - offset, y - offset)
+        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
+                                   axis=yAxis, check=True)
+        if dataPos is None:
+            return []
+        xPick0, yPick0 = dataPos
+
+        inAreaPos = self._mouseInPlotArea(x + offset, y + offset)
+        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
+                                   axis=yAxis, check=True)
+        if dataPos is None:
+            return []
+        xPick1, yPick1 = dataPos
+
+        if xPick0 < xPick1:
+            xPickMin, xPickMax = xPick0, xPick1
+        else:
+            xPickMin, xPickMax = xPick1, xPick0
+
+        if yPick0 < yPick1:
+            yPickMin, yPickMax = yPick0, yPick1
+        else:
+            yPickMin, yPickMax = yPick1, yPick0
+
+        # Apply log scale if axis is log
+        if self._plotFrame.xAxis.isLog:
+            xPickMin = numpy.log10(xPickMin)
+            xPickMax = numpy.log10(xPickMax)
+
+        if (yAxis == 'left' and self._plotFrame.yAxis.isLog) or (
+                yAxis == 'right' and self._plotFrame.y2Axis.isLog):
+            yPickMin = numpy.log10(yPickMin)
+            yPickMax = numpy.log10(yPickMax)
+
+        return item.pick(xPickMin, yPickMin,
+                         xPickMax, yPickMax)
+
     def pickItems(self, x, y, kinds):
         picked = []
 
@@ -1314,56 +1308,20 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                             picked.append(dict(kind='image',
                                                legend=item.info['legend']))
 
-                    elif 'curve' in kinds and isinstance(item, GLPlotCurve2D):
-                        offset = self._PICK_OFFSET
-                        if item.marker is not None:
-                            offset = max(item.markerSize / 2., offset)
-                        if item.lineStyle is not None:
-                            offset = max(item.lineWidth / 2., offset)
-
-                        yAxis = item.info['yAxis']
-
-                        inAreaPos = self._mouseInPlotArea(x - offset, y - offset)
-                        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
-                                                   axis=yAxis, check=True)
-                        if dataPos is None:
-                            continue
-                        xPick0, yPick0 = dataPos
-
-                        inAreaPos = self._mouseInPlotArea(x + offset, y + offset)
-                        dataPos = self.pixelToData(inAreaPos[0], inAreaPos[1],
-                                                   axis=yAxis, check=True)
-                        if dataPos is None:
-                            continue
-                        xPick1, yPick1 = dataPos
-
-                        if xPick0 < xPick1:
-                            xPickMin, xPickMax = xPick0, xPick1
-                        else:
-                            xPickMin, xPickMax = xPick1, xPick0
-
-                        if yPick0 < yPick1:
-                            yPickMin, yPickMax = yPick0, yPick1
-                        else:
-                            yPickMin, yPickMax = yPick1, yPick0
-
-                        # Apply log scale if axis is log
-                        if self._plotFrame.xAxis.isLog:
-                            xPickMin = numpy.log10(xPickMin)
-                            xPickMax = numpy.log10(xPickMax)
-
-                        if (yAxis == 'left' and self._plotFrame.yAxis.isLog) or (
-                                yAxis == 'right' and self._plotFrame.y2Axis.isLog):
-                            yPickMin = numpy.log10(yPickMin)
-                            yPickMax = numpy.log10(yPickMax)
-
-                        pickedIndices = item.pick(xPickMin, yPickMin,
-                                                  xPickMax, yPickMax)
-                        if pickedIndices:
-                            picked.append(dict(kind='curve',
-                                               legend=item.info['legend'],
-                                               indices=pickedIndices))
+                    elif 'curve' in kinds:
+                        if isinstance(item, GLPlotCurve2D):
+                            pickedIndices = self.__pickCurves(item, x, y)
+                            if pickedIndices:
+                                picked.append(dict(kind='curve',
+                                                   legend=item.info['legend'],
+                                                   indices=pickedIndices))
 
+                        elif isinstance(item, GLPlotTriangles):
+                            pickedIndices = item.pick(*dataPos)
+                            if pickedIndices:
+                                picked.append(dict(kind='curve',
+                                                   legend=item.info['legend'],
+                                                   indices=pickedIndices))
         return picked
 
     # Update curve
@@ -1441,37 +1399,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
             if label:
                 _logger.warning('Right axis label not implemented')
 
-    # Non orthogonal axes
-
-    def setBaseVectors(self, x=(1., 0.), y=(0., 1.)):
-        """Set base vectors.
-
-        Useful for non-orthogonal axes.
-        If an axis is in log scale, skew is applied to log transformed values.
-
-        Base vector does not work well with log axes, to investi
-        """
-        if x != (1., 0.) and y != (0., 1.):
-            if self._plotFrame.xAxis.isLog:
-                _logger.warning("setBaseVectors disables X axis logarithmic.")
-                self.setXAxisLogarithmic(False)
-            if self._plotFrame.yAxis.isLog:
-                _logger.warning("setBaseVectors disables Y axis logarithmic.")
-                self.setYAxisLogarithmic(False)
-
-            if self.isKeepDataAspectRatio():
-                _logger.warning("setBaseVectors disables keepDataAspectRatio.")
-                self.keepDataAspectRatio(False)
-
-        self._plotFrame.baseVectors = x, y
-
-    def getBaseVectors(self):
-        return self._plotFrame.baseVectors
-
-    def isDefaultBaseVectors(self):
-        return self._plotFrame.baseVectors == \
-            self._plotFrame.DEFAULT_BASE_VECTORS
-
     # Graph limits
 
     def _setDataRanges(self, xlim=None, ylim=None, y2lim=None):
@@ -1486,26 +1413,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         # Update axes range with a clipped range if too wide
         self._plotFrame.setDataRanges(xlim, ylim, y2lim)
 
-        if not self.isDefaultBaseVectors():
-            # Update axes range with axes bounds in data coords
-            plotLeft, plotTop, plotWidth, plotHeight = \
-                self.getPlotBoundsInPixels()
-
-            self._plotFrame.xAxis.dataRange = sorted([
-                self.pixelToData(x, y, axis='left', check=False)[0]
-                for (x, y) in ((plotLeft, plotTop + plotHeight),
-                               (plotLeft + plotWidth, plotTop + plotHeight))])
-
-            self._plotFrame.yAxis.dataRange = sorted([
-                self.pixelToData(x, y, axis='left', check=False)[1]
-                for (x, y) in ((plotLeft, plotTop + plotHeight),
-                               (plotLeft, plotTop))])
-
-            self._plotFrame.y2Axis.dataRange = sorted([
-                self.pixelToData(x, y, axis='right', check=False)[1]
-                for (x, y) in ((plotLeft + plotWidth, plotTop + plotHeight),
-                               (plotLeft + plotWidth, plotTop))])
-
     def _ensureAspectRatio(self, keepDim=None):
         """Update plot bounds in order to keep aspect ratio.
 
@@ -1619,11 +1526,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                 _logger.warning(
                     "KeepDataAspectRatio is ignored with log axes")
 
-            if flag and not self.isDefaultBaseVectors():
-                _logger.warning(
-                    "setXAxisLogarithmic ignored because baseVectors are set")
-                return
-
             self._plotFrame.xAxis.isLog = flag
 
     def setYAxisLogarithmic(self, flag):
@@ -1633,11 +1535,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
                 _logger.warning(
                     "KeepDataAspectRatio is ignored with log axes")
 
-            if flag and not self.isDefaultBaseVectors():
-                _logger.warning(
-                    "setYAxisLogarithmic ignored because baseVectors are set")
-                return
-
             self._plotFrame.yAxis.isLog = flag
             self._plotFrame.y2Axis.isLog = flag
 
@@ -1658,9 +1555,6 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
         if flag and (self._plotFrame.xAxis.isLog or
                      self._plotFrame.yAxis.isLog):
             _logger.warning("KeepDataAspectRatio is ignored with log axes")
-        if flag and not self.isDefaultBaseVectors():
-            _logger.warning(
-                "keepDataAspectRatio ignored because baseVectors are set")
 
         self._keepDataAspectRatio = flag
 
@@ -1723,3 +1617,11 @@ class BackendOpenGL(BackendBase.BackendBase, glu.OpenGLWidget):
     def setAxesDisplayed(self, displayed):
         BackendBase.BackendBase.setAxesDisplayed(self, displayed)
         self._plotFrame.displayed = displayed
+
+    def setForegroundColors(self, foregroundColor, gridColor):
+        self._plotFrame.foregroundColor = foregroundColor
+        self._plotFrame.gridColor = gridColor
+
+    def setBackgroundColors(self, backgroundColor, dataBackgroundColor):
+        self._backgroundColor = backgroundColor
+        self._dataBackgroundColor = dataBackgroundColor
