@@ -65,7 +65,7 @@ class RegionOfInterest(qt.QObject):
         # Avoid circular dependancy
         from ..tools import roi as roi_tools
         assert parent is None or isinstance(parent, roi_tools.RegionOfInterestManager)
-        super(RegionOfInterest, self).__init__(parent)
+        qt.QObject.__init__(self, parent)
         self._color = rgba('red')
         self._items = WeakList()
         self._editAnchors = WeakList()
@@ -73,6 +73,7 @@ class RegionOfInterest(qt.QObject):
         self._label = ''
         self._labelItem = None
         self._editable = False
+        self._visible = True
 
     def __del__(self):
         # Clean-up plot items
@@ -108,7 +109,7 @@ class RegionOfInterest(qt.QObject):
         return qt.QColor.fromRgbF(*self._color)
 
     def _getAnchorColor(self, color):
-        """Returns the anchor color from the base ROI  color
+        """Returns the anchor color from the base ROI color
 
         :param Union[numpy.array,Tuple,List]: color
         :rtype: Union[numpy.array,Tuple,List]
@@ -176,6 +177,34 @@ class RegionOfInterest(qt.QObject):
             # This can be avoided once marker.setDraggable is public
             self._createPlotItems()
 
+    def isVisible(self):
+        """Returns whether the ROI is visible in the plot.
+
+        .. note::
+            This does not take into account whether or not the plot
+            widget itself is visible (unlike :meth:`QWidget.isVisible` which
+            checks the visibility of all its parent widgets up to the window)
+
+        :rtype: bool
+        """
+        return self._visible
+
+    def setVisible(self, visible):
+        """Set whether the plot items associated with this ROI are
+        visible in the plot.
+
+        :param bool visible: True to show the ROI in the plot, False to
+            hide it.
+        """
+        visible = bool(visible)
+        if self._visible == visible:
+            return
+        self._visible = visible
+        if self._labelItem is not None:
+            self._labelItem.setVisible(visible)
+        for item in self._items + self._editAnchors:
+            item.setVisible(visible)
+
     def _getControlPoints(self):
         """Returns the current ROI control points.
 
@@ -209,7 +238,7 @@ class RegionOfInterest(qt.QObject):
     def setFirstShapePoints(self, points):
         """"Initialize the ROI using the points from the first interaction.
 
-        This interaction is constains by the plot API and only supports few
+        This interaction is constrained by the plot API and only supports few
         shapes.
         """
         points = self._createControlPointsFromFirstShape(points)
@@ -292,12 +321,14 @@ class RegionOfInterest(qt.QObject):
             if self._labelItem is not None:
                 self._labelItem._setLegend(legendPrefix + "label")
                 plot._add(self._labelItem)
+                self._labelItem.setVisible(self.isVisible())
 
         self._items = WeakList()
         plotItems = self._createShapeItems(controlPoints)
         for item in plotItems:
             item._setLegend(legendPrefix + str(itemIndex))
             plot._add(item)
+            item.setVisible(self.isVisible())
             self._items.append(item)
             itemIndex += 1
 
@@ -309,6 +340,7 @@ class RegionOfInterest(qt.QObject):
             for index, item in enumerate(plotItems):
                 item._setLegend(legendPrefix + str(itemIndex))
                 item.setColor(color)
+                item.setVisible(self.isVisible())
                 plot._add(item)
                 item.sigItemChanged.connect(functools.partial(
                     self._controlPointAnchorChanged, index))
@@ -410,6 +442,13 @@ class RegionOfInterest(qt.QObject):
                 plot._remove(item)
         self._labelItem = None
 
+    def _updated(self, event=None, checkVisibility=True):
+        """Implement Item mix-in update method by updating the plot items
+
+        See :class:`~silx.gui.plot.items.Item._updated`
+        """
+        self._createPlotItems()
+
     def __str__(self):
         """Returns parameters of the ROI as a string."""
         points = self._getControlPoints()
@@ -417,7 +456,7 @@ class RegionOfInterest(qt.QObject):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class PointROI(RegionOfInterest):
+class PointROI(RegionOfInterest, items.SymbolMixIn):
     """A ROI identifying a point in a 2D plot."""
 
     _kind = "Point"
@@ -426,6 +465,10 @@ class PointROI(RegionOfInterest):
     _plotShape = "point"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.SymbolMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def getPosition(self):
         """Returns the position of this ROI
 
@@ -458,6 +501,8 @@ class PointROI(RegionOfInterest):
         marker.setPosition(points[0][0], points[0][1])
         marker.setText(self.getLabel())
         marker.setColor(rgba(self.getColor()))
+        marker.setSymbol(self.getSymbol())
+        marker.setSymbolSize(self.getSymbolSize())
         marker._setDraggable(False)
         return [marker]
 
@@ -466,6 +511,8 @@ class PointROI(RegionOfInterest):
         marker.setPosition(points[0][0], points[0][1])
         marker.setText(self.getLabel())
         marker._setDraggable(self.isEditable())
+        marker.setSymbol(self.getSymbol())
+        marker.setSymbolSize(self.getSymbolSize())
         return [marker]
 
     def __str__(self):
@@ -474,7 +521,7 @@ class PointROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class LineROI(RegionOfInterest):
+class LineROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying a line in a 2D plot.
 
     This ROI provides 1 anchor for each boundary of the line, plus an center
@@ -487,16 +534,20 @@ class LineROI(RegionOfInterest):
     _plotShape = "line"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def _createControlPointsFromFirstShape(self, points):
         center = numpy.mean(points, axis=0)
         controlPoints = numpy.array([points[0], points[1], center])
         return controlPoints
 
     def setEndPoints(self, startPoint, endPoint):
-        """Set this line location using the endding points
+        """Set this line location using the ending points
 
         :param numpy.ndarray startPoint: Staring bounding point of the line
-        :param numpy.ndarray endPoint: Endding bounding point of the line
+        :param numpy.ndarray endPoint: Ending bounding point of the line
         """
         assert(startPoint.shape == (2,) and endPoint.shape == (2,))
         shapePoints = numpy.array([startPoint, endPoint])
@@ -535,6 +586,8 @@ class LineROI(RegionOfInterest):
         item.setColor(rgba(self.getColor()))
         item.setFill(False)
         item.setOverlay(True)
+        item.setLineStyle(self.getLineStyle())
+        item.setLineWidth(self.getLineWidth())
         return [item]
 
     def _createAnchorItems(self, points):
@@ -582,7 +635,7 @@ class LineROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class HorizontalLineROI(RegionOfInterest):
+class HorizontalLineROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying an horizontal line in a 2D plot."""
 
     _kind = "HLine"
@@ -591,6 +644,10 @@ class HorizontalLineROI(RegionOfInterest):
     _plotShape = "hline"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def _createControlPointsFromFirstShape(self, points):
         points = numpy.array([(float('nan'), points[0, 1])],
                              dtype=numpy.float64)
@@ -636,6 +693,8 @@ class HorizontalLineROI(RegionOfInterest):
         marker.setText(self.getLabel())
         marker.setColor(rgba(self.getColor()))
         marker._setDraggable(False)
+        marker.setLineWidth(self.getLineWidth())
+        marker.setLineStyle(self.getLineStyle())
         return [marker]
 
     def _createAnchorItems(self, points):
@@ -643,6 +702,8 @@ class HorizontalLineROI(RegionOfInterest):
         marker.setPosition(points[0][0], points[0][1])
         marker.setText(self.getLabel())
         marker._setDraggable(self.isEditable())
+        marker.setLineWidth(self.getLineWidth())
+        marker.setLineStyle(self.getLineStyle())
         return [marker]
 
     def __str__(self):
@@ -651,7 +712,7 @@ class HorizontalLineROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class VerticalLineROI(RegionOfInterest):
+class VerticalLineROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying a vertical line in a 2D plot."""
 
     _kind = "VLine"
@@ -660,6 +721,10 @@ class VerticalLineROI(RegionOfInterest):
     _plotShape = "vline"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def _createControlPointsFromFirstShape(self, points):
         points = numpy.array([(points[0, 0], float('nan'))],
                              dtype=numpy.float64)
@@ -705,6 +770,8 @@ class VerticalLineROI(RegionOfInterest):
         marker.setText(self.getLabel())
         marker.setColor(rgba(self.getColor()))
         marker._setDraggable(False)
+        marker.setLineWidth(self.getLineWidth())
+        marker.setLineStyle(self.getLineStyle())
         return [marker]
 
     def _createAnchorItems(self, points):
@@ -712,6 +779,8 @@ class VerticalLineROI(RegionOfInterest):
         marker.setPosition(points[0][0], points[0][1])
         marker.setText(self.getLabel())
         marker._setDraggable(self.isEditable())
+        marker.setLineWidth(self.getLineWidth())
+        marker.setLineStyle(self.getLineStyle())
         return [marker]
 
     def __str__(self):
@@ -720,7 +789,7 @@ class VerticalLineROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class RectangleROI(RegionOfInterest):
+class RectangleROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying a rectangle in a 2D plot.
 
     This ROI provides 1 anchor for each corner, plus an anchor in the
@@ -733,6 +802,10 @@ class RectangleROI(RegionOfInterest):
     _plotShape = "rectangle"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def _createControlPointsFromFirstShape(self, points):
         point0 = points[0]
         point1 = points[1]
@@ -838,6 +911,8 @@ class RectangleROI(RegionOfInterest):
         item.setColor(rgba(self.getColor()))
         item.setFill(False)
         item.setOverlay(True)
+        item.setLineStyle(self.getLineStyle())
+        item.setLineWidth(self.getLineWidth())
         return [item]
 
     def _createAnchorItems(self, points):
@@ -894,7 +969,7 @@ class RectangleROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class PolygonROI(RegionOfInterest):
+class PolygonROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying a closed polygon in a 2D plot.
 
     This ROI provides 1 anchor for each point of the polygon.
@@ -906,6 +981,10 @@ class PolygonROI(RegionOfInterest):
     _plotShape = "polygon"
     """Plot shape which is used for the first interaction"""
 
+    def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
+        RegionOfInterest.__init__(self, parent=parent)
+
     def getPoints(self):
         """Returns the list of the points of this polygon.
 
@@ -948,6 +1027,8 @@ class PolygonROI(RegionOfInterest):
             item.setColor(rgba(self.getColor()))
             item.setFill(False)
             item.setOverlay(True)
+            item.setLineStyle(self.getLineStyle())
+            item.setLineWidth(self.getLineWidth())
             return [item]
 
     def _createAnchorItems(self, points):
@@ -967,7 +1048,7 @@ class PolygonROI(RegionOfInterest):
         return "%s(%s)" % (self.__class__.__name__, params)
 
 
-class ArcROI(RegionOfInterest):
+class ArcROI(RegionOfInterest, items.LineMixIn):
     """A ROI identifying an arc of a circle with a width.
 
     This ROI provides 3 anchors to control the curvature, 1 anchor to control
@@ -986,6 +1067,7 @@ class ArcROI(RegionOfInterest):
                                                           'startAngle', 'endAngle'])
 
     def __init__(self, parent=None):
+        items.LineMixIn.__init__(self)
         RegionOfInterest.__init__(self, parent=parent)
         self._geometry = None
 
@@ -1211,13 +1293,13 @@ class ArcROI(RegionOfInterest):
     def getGeometry(self):
         """Returns a tuple containing the geometry of this ROI
 
-        It is a symetric fonction of :meth:`setGeometry`.
+        It is a symmetric function of :meth:`setGeometry`.
 
         If `startAngle` is smaller than `endAngle` the rotation is clockwise,
         else the rotation is anticlockwise.
 
         :rtype: Tuple[numpy.ndarray,float,float,float,float]
-        :raise ValueError: In case the ROI can't be representaed as section of
+        :raise ValueError: In case the ROI can't be represented as section of
             a circle
         """
         geometry = self._getInternalGeometry()
@@ -1357,6 +1439,8 @@ class ArcROI(RegionOfInterest):
         item.setColor(rgba(self.getColor()))
         item.setFill(False)
         item.setOverlay(True)
+        item.setLineStyle(self.getLineStyle())
+        item.setLineWidth(self.getLineWidth())
         return [item]
 
     def _createAnchorItems(self, points):
