@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2018 European Synchrotron Radiation Facility
+# Copyright (c) 2018-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@ __date__ = "28/06/2018"
 
 
 import collections
+import enum
 import functools
 import logging
 import time
@@ -38,7 +39,6 @@ import weakref
 
 import numpy
 
-from ....third_party import enum
 from ....utils.weakref import WeakMethodProxy
 from ... import qt, icons
 from .. import PlotWidget
@@ -106,6 +106,9 @@ class RegionOfInterestManager(qt.QObject):
         self._rois = []  # List of ROIs
         self._drawnROI = None  # New ROI being currently drawn
 
+        # Handle unique selection of interaction mode action
+        self._actionGroup = qt.QActionGroup(self)
+
         self._roiClass = None
         self._color = rgba('red')
 
@@ -158,6 +161,8 @@ class RegionOfInterestManager(qt.QObject):
             action.setChecked(self.getCurrentInteractionModeRoiClass() is roiClass)
             action.setToolTip(text)
 
+            self._actionGroup.addAction(action)
+
             action.triggered[bool].connect(functools.partial(
                 WeakMethodProxy(self._modeActionTriggered), roiClass=roiClass))
             self._modeActions[roiClass] = action
@@ -171,9 +176,6 @@ class RegionOfInterestManager(qt.QObject):
         """
         if checked:
             self.start(roiClass)
-        else:  # Keep action checked
-            action = self.sender()
-            action.setChecked(True)
 
     def _updateModeActions(self):
         """Check/Uncheck action corresponding to current mode"""
@@ -781,9 +783,9 @@ class RegionOfInterestTableWidget(qt.QTableWidget):
         super(RegionOfInterestTableWidget, self).__init__(parent)
         self._roiManagerRef = None
 
-        self.setColumnCount(5)
-        self.setHorizontalHeaderLabels(
-            ['Label', 'Edit', 'Kind', 'Coordinates', ''])
+        headers = ['Label', 'Edit', 'Kind', 'Coordinates', '']
+        self.setColumnCount(len(headers))
+        self.setHorizontalHeaderLabels(headers)
 
         horizontalHeader = self.horizontalHeader()
         horizontalHeader.setDefaultAlignment(qt.Qt.AlignLeft)
@@ -806,12 +808,19 @@ class RegionOfInterestTableWidget(qt.QTableWidget):
 
         self.itemChanged.connect(self.__itemChanged)
 
-    @staticmethod
-    def __itemChanged(item):
+    def __itemChanged(self, item):
         """Handle item updates"""
         column = item.column()
-        roi = item.data(qt.Qt.UserRole)
+        index = item.data(qt.Qt.UserRole)
+
+        if index is not None:
+            manager = self.getRegionOfInterestManager()
+            roi = manager.getRois()[index]
+        else:
+            return
+
         if column == 0:
+            roi.setVisible(item.checkState() == qt.Qt.Checked)
             roi.setLabel(item.text())
         elif column == 1:
             roi.setEditable(
@@ -878,17 +887,19 @@ class RegionOfInterestTableWidget(qt.QTableWidget):
         for index, roi in enumerate(rois):
             baseFlags = qt.Qt.ItemIsSelectable | qt.Qt.ItemIsEnabled
 
-            # Label
+            # Label and visible
             label = roi.getLabel()
             item = qt.QTableWidgetItem(label)
-            item.setFlags(baseFlags | qt.Qt.ItemIsEditable)
-            item.setData(qt.Qt.UserRole, roi)
+            item.setFlags(baseFlags | qt.Qt.ItemIsEditable | qt.Qt.ItemIsUserCheckable)
+            item.setData(qt.Qt.UserRole, index)
+            item.setCheckState(
+                qt.Qt.Checked if roi.isVisible() else qt.Qt.Unchecked)
             self.setItem(index, 0, item)
 
             # Editable
             item = qt.QTableWidgetItem()
             item.setFlags(baseFlags | qt.Qt.ItemIsUserCheckable)
-            item.setData(qt.Qt.UserRole, roi)
+            item.setData(qt.Qt.UserRole, index)
             item.setCheckState(
                 qt.Qt.Checked if roi.isEditable() else qt.Qt.Unchecked)
             self.setItem(index, 1, item)
