@@ -36,29 +36,16 @@ import shutil
 
 _logger = logging.getLogger(__name__)
 
+import fabio
+import h5py
 
-try:
-    import fabio
-except ImportError:
-    fabio = None
-
-try:
-    import h5py
-except ImportError:
-    h5py = None
-
-if fabio is not None and h5py is not None:
-    from .. import fabioh5
-    from .. import commonh5
+from .. import commonh5
+from .. import fabioh5
 
 
 class TestFabioH5(unittest.TestCase):
 
     def setUp(self):
-        if fabio is None:
-            self.skipTest("fabio is needed")
-        if h5py is None:
-            self.skipTest("h5py is needed")
 
         header = {
             "integer": "-100",
@@ -381,10 +368,6 @@ class TestFabioH5MultiFrames(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        if fabio is None:
-            raise unittest.SkipTest("fabio is needed")
-        if h5py is None:
-            raise unittest.SkipTest("h5py is needed")
 
         names = ["A", "B", "C", "D"]
         values = [["32000", "-10", "5.0", "1"],
@@ -470,10 +453,6 @@ class TestFabioH5WithEdf(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        if fabio is None:
-            raise unittest.SkipTest("fabio is needed")
-        if h5py is None:
-            raise unittest.SkipTest("h5py is needed")
 
         cls.tmp_directory = tempfile.mkdtemp()
 
@@ -510,21 +489,16 @@ class TestFabioH5WithEdf(unittest.TestCase):
         self.assertNotIn("/scan_0/instrument/detector_0/others/HeaderID", self.h5_image)
 
 
-if fabio is not None:
-    class _TestableFrameData(fabioh5.FrameData):
-        """Allow to test if the full data is reached."""
-        def _create_data(self):
-            raise RuntimeError("Not supposed to be called")
+class _TestableFrameData(fabioh5.FrameData):
+    """Allow to test if the full data is reached."""
+    def _create_data(self):
+        raise RuntimeError("Not supposed to be called")
 
 
 class TestFabioH5WithFileSeries(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        if fabio is None:
-            raise unittest.SkipTest("fabio is needed")
-        if h5py is None:
-            raise unittest.SkipTest("h5py is needed")
 
         cls.tmp_directory = tempfile.mkdtemp()
 
